#!/bin/bash
#
#	$Id: Buchen.sh,v 1.3 1998/03/04 02:04:16 moritz Exp $
#		(C) 1996,97 DFG/M.Watermann, Hannover, FRG
#
#	Script zum Erzeugen von Buchungs-Saetzen fuer Zerb-Buchung
#		aus den modifizierten UnixConnect-Logfiles
#
#		Aufruf: Buchen.sh import | export
#
#-----------------------------------------------------------------------------
set -h -p

#-----------------------------------------------------------------------------
#	Konfigurations-Variablen:
#
# Pfennig-Preis pro angefangenes KByte
declare -ri KBpf=1
#
# Name der Datei mit weiteren Konfigurations-Angaben:
KonfigDat="/home/helpers/mbox/zconnect/gate.vars"

#-----------------------------------------------------------------------------
#	(you shouldn't change anything beyond this point!)
#
#	$Log: Buchen.sh,v $
#	Revision 1.3  1998/03/04 02:04:16  moritz
#	neue Version
#	
#	Revision 1.8  1997/11/22 19:47:11  matthias
#	Konfigurations-Datei konfigurierbar gemacht
#
#	Revision 1.7  1996/09/20 16:04:20  matthias
#	Anpassung an unix-connect v074b4MB04;
#
#	Revision 1.6  1996/09/11 22:22:56  matthias
#	bug fix: fehlende '{' in Preis-Pruefung ergaenzt;
#
#	Revision 1.5  1996/09/11 18:49:03  matthias
#	unbenutzte Variablen usw. entfernt;
#	bei KB-Preis kleiner 1 wird sofort terminiert;
#	Namenserzeugung auf "Sekunden seit 1970" umgestellt;
#	an /usr/lib/smail/savelog wird jetzt ein Circle von 99 uebergeben;
#	Crontab-Beispiel am Ende angehaengt;
#
#	Revision 1.4  1996/09/06 12:13:06  matthias
#	globale Variablen R/O gemacht;
#	"^M" gegen "\r" ausgetauscht;
#
#	Revision 1.3  1996/08/10 23:48:04  matthias
#	Backup der Gateway-Logs realisiert;
#	Verzeichnis-Zugriffe ausgerichtet an "/home/mbox/...";
#	fehlendes (DOS-)CR am Ende der Buchungssaetze ergaenzt;
#	Netware-Datei wird jetzt unter unique-Dateinamen erzeugt;
#	Variable "Verbose" steuert Erzeugung von Copy-Meldungen;
#
#	Revision 1.2  1996/08/09 18:16:53  matthias
#	unterschiedliche Behandlung der Im- und Export-Logs;
#	einige Konfig-Variablen werden aus der gleichen Datei
#	eingelesen, die auch das Gateway-Script verwendet;
#
#	Revision 1.1  1996/08/08 21:46:00  matthias
#	Initial revision; Aufnahme ins RCS
#
#-----------------------------------------------------------------------------
#	interne Arbeitsvariablen:
GateLog=${1:?"Aufruf: ${0##*/} import | export"}
KonfigDat="${KonfigDat:-/home/helpers/mbox/zconnect/gate.vars}"
[ -s ${KonfigDat} ] || exit 1
. ${KonfigDat}					# Gateway-Variablen einlesen
declare -r BuchDatei="mailbuch.dat"
declare -r TmpFiles="/tmp/${Me}.${MyPID}"
declare -r Prefix="${TmpFiles}.Zeile."
declare -r BackTmpDir="${Spool}/tmp"		# 'Spool' steht in "gate.vars"
declare -r OutFile="${TmpFiles}.Out"
declare -r InFile="${TmpFiles}.In"
declare -r ByteMuster=" Bytes]"	# Zeichenfolge aus modifiziertem Logfile-Format
declare -r LClientFlag="${Lin2ZerbDIR}/${ClientFlag}"
declare -r IFSsave="${IFS}"
ImEx="I"

#-----------------------------------------------------------------------------
#
function DomainCheck() {
local AktDomain="`grep -i "@${2}" domains.dat`"
  [ -z "${AktDomain}" ] && return 1
  return 0
} # DomainCheck

#-----------------------------------------------------------------------------
#
function SchreibS1() {
  echo -n "${1}	" >>${OutFile}
} # SchreibS1 mit TAB

#-----------------------------------------------------------------------------
#
function SchreibS2() {
  echo -n "${1}, " >>${OutFile}
} # SchreibS2 mit Komma/Space

#-----------------------------------------------------------------------------
#
function HandleLine() {
#
local Datum="${1}"  ; local Zeit="${2}"
# local ABS="${3%%]}" ; local LEN="${4%%$ByteMuster}"		# v.074b4MB01
# local EMP="${5%%]}" ; local MID="${6%%]}" ; local OK		# v.074b4MB01
local ABS="${3%]}" ; local EMP="${4%]}"
local MID="${5%]}" ; local LEN="${6%$ByteMuster}"
local KBytePreisPfg ; local KBytePreisDM
local KBytes=0 ; local KBytePreis=0
local OK
  [ -z ${Datum} ] && return 1				# leere Zeile: wech hier
  ABS="${ABS#[}"
  EMP="${EMP#[}"
  # Domain-Pruefung:
  IFS=" @
	"						# Space, @, TAB und LF
  if [ "${ImEx}" = "I" ] ; then
    DomainCheck ${ABS}
  else
    DomainCheck ${EMP}
  fi
  OK=$?
  IFS="${IFSsav}"
  [ ${OK} -ne 0 ] && return 0		# keine abzurechnende Domain gefunden
  MID="${MID#[}"
  LEN="${LEN#[}"
  # Preis-Berechnung:
  let KBytes=$[${LEN}>>10]
  [ ${KBytes} -lt 1 ] && KBytes=1
  let KBytePreis=$[ ${KBytes} * ${KBpf} ]
  let KBytePreisPfg=$[ ${KBytePreis} % 100 ]
  let KBytePreisDM=$[ ${KBytePreis} / 100]
  if [ ${KBytePreisPfg} -lt 10 ] ; then
    KBytePreis="${KBytePreisDM},0${KBytePreisPfg}"
  else
    KBytePreis="${KBytePreisDM},${KBytePreisPfg}"
  fi
  [ ${KBytePreisDM} -lt 10 ] && KBytePreis=" ${KBytePreis}"
  IFS=" @
	"						# Space, @, TAB und LF
  if [ "${ImEx}" = "I" ] ; then
    SchreibS1 ${ABS}
    echo -n "${KBytePreis}	EMail: (${LEN} Bytes ${Datum}/${Zeit}) an "\
    >>${OutFile}
    IFS=" 
	"						# Space, LF und TAB
    SchreibS2 ${EMP}
    echo -e "Msg-ID: <${MID}>\r">>${OutFile}
  else
    SchreibS1 ${EMP}
    echo -n "${KBytePreis}	EMail: (${LEN} Bytes ${Datum}/${Zeit}) von "\
    >>${OutFile}
    IFS=" 
	"						# Space, LF und TAB
    SchreibS2 ${ABS}
    echo -e "Msg-ID: ${MID}\r">>${OutFile}
  fi
  IFS="${IFSsav}"
  return 0
} # HandleLine

#-----------------------------------------------------------------------------
#
function ReadLines() {
#
local F1 ; local F2 ; local F3
local F4 ; local F5 ; local REST
local OK ; local Zeile
  split -1 ${InFile} ${Prefix}		# Datei in Zeilen zerlegen
  for Zeile in ${Prefix}* ; do
    IFS="
	"				# nur TAB und LF als Feld-Separator
    read F1 F2 F3 F4 F5 REST <${Zeile}
    OK=$?
    IFS="${IFSsave}"
    [ \( ${OK} -ne 0 \) -o \( -z "${F1}" \) ] && return 1
    HandleLine "${F1}" "${F2}" "${F3}" "${F4}" "${F5}" "${REST}"
    [ $? -eq 0 ] || return 1
  done
  # wir benutzen ein Script aus der SMail-Distribution zum Sichern der Datei:
  [ -s ${OutFile} ] && /usr/lib/smail/savelog -t -c 99 ${GateLog} &>/dev/null
  rm -f ${Prefix}*
  return 0
} # ReadLines

#-----------------------------------------------------------------------------
#
function SichereBuchung() {
#
local BackUpFile="`date +%s`.0"
  [ -s ${OutFile} ] || return 0				# nix da zum Sichern
  [ -d ${BackTmpDir} ] || mkdir -p ${BackTmpDir} || return 1
  if [ "${ImEx}" = "I" ] ; then
    BackUpFile="${BackTmpDir}/${BackUpFile}.import.${BuchDatei}"
  else
    BackUpFile="${BackTmpDir}/${BackUpFile}.export.${BuchDatei}"
  fi
  mv -f${Verbose} ${OutFile} ${BackUpFile}
  return 0
} # SichereBuchung

#-----------------------------------------------------------------------------
#
function Lin2Zerb() {
#
local LMasterFlag="${Lin2ZerbDIR}/${MasterFlag}"
local BLin2ZerbDIR="${Lin2ZerbDIR}/buchen" # Unterverzeichnis fuer Buchungsdaten
local OldFile ; local BackUpFile
local BackUpDir=${BackInDir}
local Maske1 ; local Maske2
local Secs=0
  [ -s ${OutFile} ] || return 0		# nix da zum Buchen
  [ -d ${BLin2ZerbDIR} ] || return 1	# Netware-Dir nicht zu benutzen
  [ -f ${LMasterFlag} ] && return 1	# wird vom Zerb-Cronjob erzeugt
  echo ${MyPID} >${LClientFlag}		# wird vom Zerb-Cronjob respektiert
  [ -w ${LClientFlag} ] || return 1	# Flagdatei nicht zu lesen
  # erstmal liegengebliebene Dateien verarbeiten:
  for OldFile in ${BackTmpDir}/*.${BuchDatei} ; do
    if [ -s ${OldFile} ] ; then
      Secs=`date +%s`
      while [ ${#Secs} -gt 8 ] ; do	# auf 8.3-Konvention kuerzen
	  Secs=${Secs#?}
      done
      cp -f${Verbose} ${OldFile} ${BLin2ZerbDIR}/${Secs}.dat
      let SomethingDone+=1
      Maske1="`basename ${OldFile%%.$BuchDatei}`"
      Maske2=${Maske1##import.}
      if [ "${Maske1}" = "${Maske2}" ] ; then			# export-Datei
	BackUpFile="${BackOutDir}/${Maske1##export.}"
      else							# import-Datei
	BackUpFile="${BackInDir}/${Maske2}"
      fi
      mv -f${Verbose} ${OldFile} ${BackUpFile}
      sleep 1				# zur Sicherheit eine Sekunde ausruhen
    fi
  done
  # ... und nun die aktuelle:
  [ "${ImEx}" = "E" ] && BackUpDir="${BackOutDir}"
  BackUpFile="${BackUpDir}/${Date}.${BuchDatei}"
  mv -f${Verbose} ${OutFile} ${BackUpFile}
  Secs=`date +%s`
  while [ ${#Secs} -gt 8 ] ; do		# auf 8.3-Konvention (DOS) kuerzen
    Secs=${Secs#?}
  done
  cp -f${Verbose} ${BackUpFile} ${BLin2ZerbDIR}/${Secs}.dat
  let SomethingDone+=1
  rm -f ${LClientFlag}
  return 0
} # Lin2Zerb

#-----------------------------------------------------------------------------
#
function CleanUp() {
#
  rm -f ${TmpFiles}* ${WorkFlag} ${LClientFlag} ${Flag4Both}
  IFS="${IFSsave}"
  exit 0
} # CleanUp

#-----------------------------------------------------------------------------
#
# Main()
#
ChkBothFlag || exit 1				# Gateway-Script aktiv?
trap " CleanUp " 0 1 2 3 9 15
[ -z "${GateLog}" ] && exit 1			# Parameter uebergeben?
[ ${KBpf} -lt 1 ] && exit 0			# nix zu berechnen
builtin cd ${BinDir} || exit 1			# ins Home-Verzeichnis wechseln
[ -f ${WorkFlag} ] && exit 1			# andere Programm-Kopie aktiv?
umask 007 ; echo $$ >${WorkFlag}
[ -w ${WorkFlag} ] || exit 1			# Arbeits-Flag OK?

GateLog=${GateLog##*/}
GateLog=${GateLog%%.log}
[ "${GateLog}" = "export" ] && ImEx="E"
GateLog="../logfiles/${GateLog}.log"
[ -s ${GateLog} ] || exit 1			# leere Datei ignorieren

grep "${ByteMuster}" ${GateLog} >${InFile}	# Daten in Datei sichern
[ -s ${InFile} ] || exit 1			# ist 'was angekommen?

ReadLines				# Zeilen Lesen und Buchungen schreiben
Lin2Zerb				# Buchungsdaten auf NW-Server schieben
SichereBuchung					# liegengebliebenes sichern

[ -n "${Verbose}" -a ${SomethingDone} -gt 0 ] && times
exit 0						# ... und tschuess ...

#-----------------------------------------------------------------------------
#
# Das Script ist hier in '/etc/crontab' so eingebunden:
#
# ZC-Gateway Mailkosten buchen:
51 23 * * * mbox /home/helpers/mbox/zconnect/Buchen.sh import
52 23 * * * mbox /home/helpers/mbox/zconnect/Buchen.sh export
#
# _EoF_
